/*A version of signal() that allows flags to be specified, yet easier
 *to call than sigaction(), and also more portable
 */
#include "signalx.h"

/*Just like signal(), except sigaction flags are specified
 *explicitly. The idiotic #ifdef in the middle takes care of the
 *outdated systems, such as SunOS, that define SA_INTERRUPT instead of
 *SA_RESTART, with the opposite sense. Also, on some wierd systems
 *(like linux) that define both it makes sure that exactly one of the
 *two flags is set. Actually, linux doesn't look at SA_INTERRUPT,
 *which is the default, which can be overridden by SA_RESTART. As long
 *as they don't assign SA_INTERRUPT to something else, this should
 *work. And don't ask me why people do that
 */
void (*sighandle(int signal, void (*action)(int), int flags))(int) {
  struct sigaction sainp;
  struct sigaction saout;

  sainp.sa_handler = action;
  sigemptyset(&sainp.sa_mask);
#ifndef SA_INTERRUPT
  sainp.sa_flags = flags;
#else
  if (flags & SA_RESTART) sainp.sa_flags = flags & ~SA_INTERRUPT;
  else sainp.sa_flags = flags | SA_INTERRUPT;
#endif

  if (sigaction(signal, &sainp, &saout) == -1) return SIG_ERR;
  return saout.sa_handler;
}

//Block signal
void sigdefer(int signal) {
  sigset_t samask;

  sigemptyset(&samask);
  sigaddset(&samask, signal);
  sigprocmask(SIG_BLOCK, &samask, NULL);
}

//Unblock signal
void sigresume(int signal) {
  sigset_t samask;

  sigemptyset(&samask);
  sigaddset(&samask, signal);
  sigprocmask(SIG_UNBLOCK, &samask, NULL);
}